/*
Copyright (c) 2009, Dirk Krause
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.
* Redistributions in binary form must reproduce the above 
  opyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.
* Neither the name of the Dirk Krause nor the names of
  contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
*/



package dirk_krause.jtools.application;

import java.awt.print.Printable;
import java.awt.print.PrinterJob;
import java.awt.print.PageFormat;
import java.awt.print.Paper;


/**
 * This class passes one java.awt.Printable to the print system.
 * @author krause
 */
public class Printing {
    /**
     * Print one printable.
     * @param pr    Printable to print
     * @param pw    Paper width.
     * @param ph    Paper height.
     * @param x     Start of imageable area x..
     * @param y     Start of imageable area y.
     * @param w     Width of imageable area.
     * @param h     Height of imageable area.
     * @return Flag for success (true) or failure (false).
     */
    static public boolean processPrintable(
        Printable pr,
	double pw, double ph,
	double x, double y,
	double w, double h
    )
    {
        boolean back = false;
        PrinterJob pjob = PrinterJob.getPrinterJob();
	Paper pa = new Paper();
	pa.setSize(pw, ph);
	pa.setImageableArea(x, y, w, h);
	PageFormat pf = new PageFormat();
	pf.setPaper(pa);
	pjob.setPrintable(pr, pf);
        if(pjob.printDialog()) {
            try {
                pjob.print();
                back = true;
            } catch(Exception e) { back = false; }
        }
        return back;
    }

    /**
     * Print one printable.
     * @param pr    Printable to print
     * @return Flag for success (true) or failure (false).
     */
    static public boolean processPrintable(Printable pr) {
        boolean back = false;
	back = processPrintable(
	  pr,
	  595.0, 842.0,
	  56.692913385, 28.346456693, 481.614173228, 785.307086614
	);
        return back;
    }



}


// project dklibsj
// @(#)Printing.java 1.3 12/02/09
// vim: set ai sw=4 :

