package DKrause::FigWriter;

use 5.000;
use strict;
use Carp;

require Exporter;
require DynaLoader;
use AutoLoader;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $AUTOLOAD);
@ISA = qw(Exporter
	DynaLoader);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use DKrause::FigWriter ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
%EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

@EXPORT = qw(
	
);

$VERSION = '0.01';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "&DKrause::FigWriter::constant not defined" if $constname eq 'constant';
    my ($error, $val) = constant($constname);
    if ($error) { croak $error; }
    {
	no strict 'refs';
	# Fixed between 5.005_53 and 5.005_61
#XXX	if ($] >= 5.00561) {
#XXX	    *$AUTOLOAD = sub () { $val };
#XXX	}
#XXX	else {
	    *$AUTOLOAD = sub { $val };
#XXX	}
    }
    goto &$AUTOLOAD;
}

bootstrap DKrause::FigWriter $VERSION;

# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

DKrause::FigWriter - Perl extension for writing Fig files.

=head1 SYNOPSIS

  use DKrause::FigWriter;

  my $fw1 = DKrause::FigWriter::newInches(5.0, 3.0);
  my $fw2 = DKrause::FigWriter::newCm(10.0, 7.0);

  $fw1->setLayer($layernumber);

  $fw1->layerUp();

  my $colornumber = $fw1->defineColor($r, $g, $b);

  $fw1->setPenColor($colornumber);
  $fw1->setPenColorByName($colorname);

  $fw1->setFillColor($colornumber);
  $fw1->setFillColorByName($colorname);

  $fw1->setFillNone();
  $fw1->setFillBlack();
  $fw1->setFillSaturation($saturation);
  $fw1->setFillLeftDiagonal30();
  $fw1->setFillRightDiagonal30();
  $fw1->setFillCrosshatch30();
  $fw1->setFillLeftDiagonal45();
  $fw1->setFillRightDiagonal45();
  $fw1->setFillCrosshatch45();
  $fw1->setfillHorizontalBricks();
  $fw1->setFillVerticalBricks();
  $fw1->setFillHorizontalLines();
  $fw1->setFillVerticalLines();
  $fw1->setFillCrosshatch();
  $fw1->setFillHorizontalShinglesRight();
  $fw1->setFillHorizontalShinglesLeft();
  $fw1->setFillVerticalShinglesA();
  $fw1->setFillVerticalShinglesB();
  $fw1->setFillFishScales();
  $fw1->setFillSmallFishScales();
  $fw1->setFillCircles();
  $fw1->setFillHexagons();
  $fw1->setFillOctagons();
  $fw1->setFillHorizontalTireTreads();
  $fw1->setFillVerticalTireTreads();

  $fw1->setLineStyleSolid();
  $fw1->setLineStyleDashed();
  $fw1->setLineStyleDotted();
  $fw1->setLineStyleDashDotted();
  $fw1->setLineStyleDashDoubleDotted();
  $fw1->setLineStyleDashTripleDotted();

  $fw1->setLineWidth($linewidth);

  $fw1->setLineJoinMitered();
  $fw1->setLineJoinRounded();
  $fw1->setLineJoinBeveled();

  $fw1->setLineCapButted();
  $fw1->setLineCapRounded();
  $fw1->setLineCapProjecting();

  $fw1->setArrowheadForwardStick();
  $fw1->setArrowheadForwardTriangle();
  $fw1->setArrowheadForwardIndented();
  $fw1->setArrowheadForwardPointed();

  $fw1->setArrowheadForwardFilled($fillflag);

  $fw1->setArrowheadForwardWidthHeight($w, $h);

  $fw1->setArrowheadBackwardStick();
  $fw1->setArrowheadBackwardTriangle();
  $fw1->setArrowheadBackwardIndented();
  $fw1->setArrowheadBackwardPointed();

  $fw1->setArrowheadBackwardFilled($fillflag);

  $fw1->setArrowheadBackwardWidthHeight($w, $h);

  $fw1->setArrowheadDirections($dirs);

  $fw1->rectangle($x0, $y0, $x1, $y1);

  $fw1->arcBox($x0, $y0, $x1, $y1, $r);

  $fw1->image($x0, $y0, $x1, $y1, $filename);

  $fw1->polygon();
  $fw1->point($x0, $y0);
  $fw1->point($x1, $y1);
  ...


  $fw1->polyline();
  $fw1->point($x0, $y0);
  $fw1->point($x1, $y1);
  ...

  $fw1->splineOpen();
  $fw1->splinePoint($x0, $y0, $s0);
  $fw1->splinePoint($x1, $y1, $s1);
  ...

  $fw1->splineClosed();
  $fw1->splinePoint($x0, $y0, $s0);
  $fw1->splinePoint($x1, $y1, $s1);
  ...

  $fw1->arcOpen($x, $y, $r, $alphastart, $alphaend);

  $fw1->arcClosed($x, $y, $r, $alphastart, $alphaend);

  $fw1->circle($x, $y, $r);

  $fw1->ellipse($x, $y, $rx, $ry, $rotation);

  $fw1->setTextAlignedLeft();
  $fw1->setTextAlignedCentered();
  $fw1->setTextAlignedRight();

  $fw1->setTextFlagRigid($flagvalue);

  $fw1->setTextFlagSpecial($flagvalue);

  $fw1->setTextFontPs($fontindex);
  $fw1->setTextFontPsByName($fontname);

  $fw1->setTextFontLatex($fontindex);

  $fw1->setTextSize($textsize);

  $fw1->text($x, $y, $text, $rotation);

  $fw1->setUtf8($utf8flag);

  $fw1->setTransparentColor($colornumber);

  $fw1->setAngleDegree();
  $fw1->setAngleRadians();

  $fw1->writeFile($filename);


=head1 DESCRIPTION

The module creates an in-memory representation of a graphic and writes
the graphic to a Fig file.

Fig files can be edited using GUI based programs like XFig or jFig.
You can use these programs to apply fine tuning to the created graphics
file.

Programs like fig2vect (see http://fig2vect.sourceforge.net) or
fig2dev (part of the transfig package in the XFig distribution)
can be used to convert Fig files to other graphics formats.

The newInches() or newCm() functions can be used to create a new
graphics, image width and height must be specified in inches or cm.

The origin for coordinates is the B<lower> left corner. Coordinates
values grow to the right and to top.

The main purpose of the module is to free the programmer from
cm to Fig units or inch to Fig units calculations and from
y transformations for the origin in the upper left corner.

Each graphics object is placed in a layer, large layer numbers mean
layers deep below other layers. At start the current layer is 998.
You can use setLayer() to set a new current layer by specifying a
number of layerUp() to increase the current layer number by one.

A number of 32 colors is predefined by the Fig file format. You
can define new colors using the defineColor() function if necessary,
the RGB components must be specified as integers in the range
0...255.

The setPenColor() and setPenColorByName() functions set the pen
color which is used for linestroking and text.

The setFillColor() and setFillColorByName() functions set the fill color which
is used to fill objects.

The relation between color indices and color names is shown here:

   0 "black"
   1 "blue"
   2 "green"
   3 "cyan"
   4 "red"
   5 "magenta"
   6 "yellow"
   7 "white"
   8 "blue.darkest"
   9 "blue.dark"
  10 "blue.light"
  11 "blue.lightest"
  12 "green.dark"
  13 "green.normal"
  14 "green.light"
  15 "cyan.dark"
  16 "cyan.normal"
  17 "cyan.light"
  18 "red.dark"
  19 "red.normal"
  20 "red.light"
  21 "magenta.dark"
  22 "magenta.normal"
  23 "magenta.light"
  24 "brown.dark"
  25 "brown.normal"
  26 "brown.light"
  27 "pink.darkest"
  28 "pink.dark"
  29 "pink.light"
  20 "pink.lightest"
  31 "gold"

The setFillXXX() functions set the fill style.
The setfillSaturation() function needs an integer argument
(0, 5, 10, ... 95, 100, 105, ... 200). A saturation of 100 percent means
"fill with the color". Lower saturation values mean darker shades of the color,
higher values mean lighter shades of the color.

The setLineStyleXXX() functions can be used to set different line styles.

the setLineWidth() function sets the line width. An integer argument
is expected, the line widths are multiples of 1/80 inch.

The setLineJoinXXX() functions can be used to set different
line join types.

The setLineCapXXX() functions can be used to set line end style.

The setArrowheadXXXStick() ... setArrowheadXXXPointed() function can be
used to set an arrowhead shape.

The setArrowheadXXXFilled() function sets the arrowhead fill type
(filled with line color for non-zero fill flag, white filled for 0 fill flag).

The setArrowheadXXXWidthHeight() function sets the arrowhead width and height).

The functions above set the style for subsequent graphics objects added.

The rectangle() function adds a rectangle.

The arcBox() function adds a rectangle with rounded corners, the corner
radius is specified in multiples of 1/80 inch.

The image() function add a reference to an image file.

The polygon() and polyline() functions add a polygon or a polyline.
The point coordinates can be specified using the point() function
multiple times. All points must be specified before the next graphics object
is added.

The splineOpen() and splineClosed() functions can be used to add open or
closed splines. The point coordinates and approximation/interpolation
coefficients can be specified using the splinePoint() function multiple
times. All points must be specified before the next graphics object
is added. The approximation/interpolation coefficient must be in the
range from -1.0 (interpolcation) to 1.0 (approximation).

The arcOpen() and arcClosed() functions add open arcs or closed
arcs (pie wedges).

The circle() function adds a circle, the ellipse() function adds
an ellipse.

The setTextAlignedXXX() functions set the text alignment for all
subsequent text adding operations.

The setTextFlagRigid() function sets/resets the rigid flag for all
subsequent text adding operations.

The setTextflagSpecial() function sets/resets the LaTeX special flag for all
subsequent text adding operations.

The setTextFontPS() and setTextFontPsByName() functions set a PS font for all
subsequent text adding operations. The relation between font indices and font
names is shown here:

   0 "Times Roman"
   1 "Times Italic"
   2 "Times Bold"
   3 "Times Bold Italic"
   4 "Avantgarde Book"
   5 "Avantgarde Book Oblique"
   6 "Avantgarde Demi"
   7 "Avantgarde Demi Oblique"
   8 "Bookman Light"
   9 "Bookman Light Italic"
  10 "Bookman Demi"
  11 "Bookman Demi Italic"
  12 "Courier"
  13 "Courier Oblique"
  14 "Courier Bold"
  15 "Courier Bold Oblique"
  16 "Helvetica"
  17 "Helvetica Oblique"
  18 "Helvetica Bold"
  19 "Helvetica Bold Oblique"
  20 "Helvetica Narrow"
  21 "Helvetica Narrow Oblique"
  22 "Helvetica Narrow Bold"
  23 "Helvetica Narrow Bold Oblique"
  24 "New Century Schoolbook Roman"
  25 "New Century Schoolbook Italic"
  26 "New Century Schoolbook Bold"
  27 "New Century Schoolbook Bold Italic"
  28 "Palatino Roman"
  29 "Palatino Italic"
  30 "Palatino Bold"
  31 "Palatino Bold Italic"
  32 "Symbol"
  33 "Zapf Chancery Medium Italic"
  34 "Zapf Dingbats"

The setTextFontLatex() function sets a LaTeX font for all
subsequent text add operations:

   0 default
   1 roman
   2 bold
   3 italic
   4 sans-serif
   5 typewriter

The setTextSize() function sets the text size (in pt) for all
subsequent text add operations.

The text() function adds text to the graphics.

The setUtf8() function sets/resets the UTF-8 encoding flag for the
entire drawing. If this flag is set all texts in the drawing are
treated as UTF-8 encoded.
NOTE: The Fig file format specification does not cover text encoding
at all. If your texts are UTF-8 encoded you should use fig2vect
to convert the file to other graphics formats.

The setAngleXXX() functions set the angle unit for all angles in
all graphics elements.

The writeFile() function writes the graphics to a Fig file.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Dirk Krause, E<lt>krause@suse.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009 by Dirk Krause

All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

=over	4

=item	Redistributions of source code must retain the above
copyright notice, this list of conditions and the
following disclaimer.

=item	Redistributions in binary form must reproduce the above 
opyright notice, this list of conditions and the following
disclaimer in the documentation and/or other materials
provided with the distribution.

=item	Neither the name of the Dirk Krause nor the names of
contributors may be used to endorse or promote
products derived from this software without specific
prior written permission.

=back

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

=cut
